@@ -62,19 +62,110 @@ static Lum::Def::AppInfo info;
 typedef Lum::Model::StdRefTable<Hiscore::Entry,std::vector<Hiscore::Entry> > HiscoreEntriesModel;
 typedef Lum::Base::Reference<HiscoreEntriesModel>                            HiscoreEntriesModelRef;
 
+class HiscoreEntryObject : public Lum::Object
+{
+private:
+  Lum::OS::FontRef font;
+  Hiscore::Entry   entry;
+
+public:
+  HiscoreEntryObject()
+  : font(Lum::OS::display->GetFont())
+  {
+    // no code
+  }
+
+  void SetEntry(const Hiscore::Entry& entry)
+  {
+    this->entry=entry;
+  }
+
+  void CalcSize()
+  {
+    minWidth=0;
+    minHeight=font->height;
+
+    width=minWidth;
+    height=minHeight;
+
+    Object::CalcSize();
+  }
+
+  void Draw(int x, int y, size_t w, size_t h)
+  {
+    Object::Draw(x,y,w,h);
+
+    if (!OIntersect(x,y,w,h)) {
+      return;
+    }
+
+    /* --- */
+
+    Lum::OS::DrawInfo *draw=GetDrawInfo();
+
+    draw->PushFont(font);
+    draw->PushForeground(parent->GetTextColor(draw));
+
+    //
+    // Name of User
+    //
+
+    draw->DrawString(this->x,
+                     this->y+font->ascent,
+                     entry.name);
+
+    //
+    //
+    // Time in seconds
+
+    std::wstring time=Lum::Base::NumberToWString(entry.time);
+
+    if (entry.time==1) {
+      time+=L" second";
+    }
+    else {
+      time+=L" seconds";
+    }
+
+    draw->DrawString(this->x+width-font->StringWidth(time),
+                     this->y+font->ascent,
+                     time);
+
+    draw->PopFont();
+    draw->PopForeground();
+  }
+
+  static HiscoreEntryObject* Create(bool horizFlex, bool vertFlex)
+  {
+    HiscoreEntryObject *object=new HiscoreEntryObject();
+
+    object->SetFlex(horizFlex,vertFlex);
+
+    return object;
+  }
+};
+
 class HiscoreEntryDataProvider : public HiscoreEntriesModel::DataProvider
 {
+private:
+  HiscoreEntryObject *object;
+
 public:
-  std::wstring GetString(const HiscoreEntriesModel::Iterator& iter, size_t column) const
+  HiscoreEntryDataProvider()
+   : object(HiscoreEntryObject::Create(true,false))
   {
-    switch (column) {
-    case 1:
-      return iter->name;
-    case 2:
-      return Lum::Base::NumberToWString((unsigned long)iter->time);
-    default:
-      assert(false);
-    }
+    // no code
+  }
+
+  ~HiscoreEntryDataProvider()
+  {
+    delete object;
+  }
+
+  Lum::Object* GetObject(const HiscoreEntriesModel::Iterator& iter, size_t column) const
+  {
+    object->SetEntry(*iter);
+    return object;
   }
 };
 
@@ -93,6 +184,7 @@ public:
                     sweeper->GetMines());
 
     list=new HiscoreEntriesModel(score->list,new HiscoreEntryDataProvider());
+    list->SetEmptyText(L"(No hiscores)");
   }
 
   Lum::Object* GetContent()
@@ -104,14 +196,13 @@ public:
     panel=Lum::VPanel::Create(true,true);
 
     headerModel=new Lum::Model::HeaderImpl();
-    headerModel->AddColumn(L"Name",Lum::Base::Size::stdCharWidth,18,true);
-    headerModel->AddColumn(L"Time",Lum::Base::Size::stdCharWidth,8);
+    headerModel->AddColumn(L"Score",Lum::Base::Size::stdCharWidth,26,true);
 
     table=new Lum::Table();
     table->SetFlex(true,true);
     table->SetMinWidth(Lum::Base::Size::stdCharWidth,30);
     table->SetMinHeight(Lum::Base::Size::stdCharHeight,10);
-    table->SetShowHeader(true);
+    table->SetShowHeader(false);
     table->GetTableView()->SetAutoFitColumns(true);
     table->GetTableView()->SetAutoVSize(true);
     table->SetModel(list);
@@ -225,19 +316,103 @@ public:
 typedef Lum::Model::StdRefTable<Theme,std::vector<Theme> > ThemesModel;
 typedef Lum::Base::Reference<ThemesModel>                  ThemesModelRef;
 
+class ThemeObject : public Lum::Object
+{
+private:
+  Lum::OS::FontRef font;
+  Theme            theme;
+
+public:
+  ThemeObject()
+  : font(Lum::OS::display->GetFont())
+  {
+    // no code
+  }
+
+  void SetTheme(const Theme& theme)
+  {
+    this->theme=theme;
+  }
+
+  void CalcSize()
+  {
+    minWidth=0;
+    minHeight=font->height;
+
+    width=minWidth;
+    height=minHeight;
+
+    Object::CalcSize();
+  }
+
+  void Draw(int x, int y, size_t w, size_t h)
+  {
+    Object::Draw(x,y,w,h);
+
+    if (!OIntersect(x,y,w,h)) {
+      return;
+    }
+
+    /* --- */
+
+    Lum::OS::DrawInfo *draw=GetDrawInfo();
+
+    draw->PushFont(font);
+    draw->PushForeground(parent->GetTextColor(draw));
+
+    //
+    // Name of theme
+    //
+
+    draw->DrawString(this->x,
+                     this->y+font->ascent,
+                     theme.name);
+
+    //
+    //
+    // Size of theme
+
+    std::wstring size=Lum::Base::NumberToWString(theme.boxSize)+L" pixel";
+
+    draw->DrawString(this->x+width-font->StringWidth(size),
+                     this->y+font->ascent,
+                     size);
+
+    draw->PopFont();
+    draw->PopForeground();
+  }
+
+  static ThemeObject* Create(bool horizFlex, bool vertFlex)
+  {
+    ThemeObject *object=new ThemeObject();
+
+    object->SetFlex(horizFlex,vertFlex);
+
+    return object;
+  }
+};
+
 class ThemeDataProvider : public ThemesModel::DataProvider
 {
+private:
+  ThemeObject *object;
+
 public:
-  std::wstring GetString(const ThemesModel::Iterator& iter, size_t column) const
+  ThemeDataProvider()
+  : object(ThemeObject::Create(true,false))
   {
-    switch (column) {
-    case 1:
-      return iter->name;
-    case 2:
-      return Lum::Base::NumberToWString(iter->boxSize);
-    default:
-      assert(false);
-    }
+    // no code
+  }
+
+  ~ThemeDataProvider()
+  {
+    delete object;
+  }
+
+  Lum::Object* GetObject(const ThemesModel::Iterator& iter, size_t column) const
+  {
+    object->SetTheme(*iter);
+    return object;
   }
 };
 
@@ -256,6 +431,8 @@ public:
     selection(new Lum::Model::SingleLineSelection),
     result(L"")
   {
+    list->SetEmptyText(L"(No themes found)");
+
     okAction->Disable();
 
     Observe(okAction);
@@ -273,13 +450,12 @@ public:
     panel->SetHeight(Lum::Base::Size::pixel,(GetWindow()->GetParent()->GetHeight()*80)/100);
 
     headerModel=new Lum::Model::HeaderImpl();
-    headerModel->AddColumn(L"Theme name",Lum::Base::Size::stdCharWidth,25);
-    headerModel->AddColumn(L"box size",Lum::Base::Size::stdCharWidth,13);
+    headerModel->AddColumn(L"Theme",Lum::Base::Size::stdCharWidth,20,true);
 
     table=new Lum::Table();
     table->SetFlex(true,true);
-    table->SetShowHeader(true);
-    table->GetTableView()->SetAutoHSize(true);
+    table->SetShowHeader(false);
+    table->GetTableView()->SetAutoFitColumns(true);
     table->SetModel(list);
     table->SetHeaderModel(headerModel);
     table->SetSelection(selection);
